#include <genesis.h>
#include "res_image.h"

#define SCREEN_WIDTH 320
#define SCREEN_HEIGHT 224

int main(u16 hard) {
    // 経過時間
    u8 timer = 0;
    // ロゴ(スプライト)の位置
    s16 posX = 112;
    s16 posY = 96;
    // 移動速度
    s16 speedX = 1;
    s16 speedY = 2;
    // 移動方向
    s16 directionX = 1;
    s16 directionY = 1;
    // 移動範囲
    s16 maxWidth = SCREEN_WIDTH - 144; // 144: ロゴ幅
    s16 maxHeight = SCREEN_HEIGHT - 40; // 40: ロゴ高さ

    // 色の設定
    PAL_setPaletteColors(0, &logo_palette, DMA);

    // スプライトエンジンの初期化
    SPR_init();

    // スプライトの追加と取得
    Sprite* logo = SPR_addSprite(&logo_sprite, posX, posY, TILE_ATTR(PAL0, TRUE, FALSE, FALSE));

    // メインループ
    while (TRUE) {
        SYS_doVBlankProcess();

        // 4フレームごとに更新する
        // NOTE: 「x & 3」は剰余演算「x % 4」と同じ
        //   つまり4で割った余りが0以外の場合は処理を飛ばしている
        if (timer++ & 3) {
            continue;
        }

        // スプライト更新(スプライトに対して行った変更が反映される)
        SPR_update();

        // ロゴの移動先座標の設定
        posX += directionX * speedX;
        posY += directionY * speedY;

        // ロゴを移動
        SPR_setPosition(logo, posX, posY);

        // 画面端であれば移動方向を反転させておく
        if (posX <= 0 || posX >= maxWidth) {
            directionX *= -1;
        }
        if (posY <= 0 || posY >= maxHeight) {
            directionY *= -1;
        }
    }

    return 0;
}